@@ -17,6 +17,8 @@ extern "C"
 {
 #endif
 
+#define SVGALIB_VER 0x010922
+
 #define TEXT 	     0		/* Compatible with VGAlib v1.2 */
 #define G320x200x16  1
 #define G640x200x16  2
@@ -193,9 +195,17 @@ extern "C"
 #define G400x600x16M 143
 #define G400x600x16M32 144
 
-#define __GLASTMODE G400x600x16M32
+#define G400x300x256X 145
+
+#define G320x200x256V 146
+
+#define __GLASTMODE G320x200x256V
 #define GLASTMODE vga_lastmodenumber()
 
+#define IS_IN_STANDARD_VGA_DRIVER(mode) ( \
+	((mode) < G640x480x256) || ((mode) == G720x348x2) || \
+	( ((mode) >= G400x300x256X) && ((mode) <= G400x300x256X) ) )
+
     extern int vga_version;
 
     extern int vga_setmode(int mode);
@@ -240,20 +250,23 @@ extern "C"
 #define HAVE_BLITWAIT 16
 
 /* other flags */
-#define HAVE_RWPAGE 1		/* vga_setreadpage() / vga_setwritepage() available */
-#define IS_INTERLACED 2		/* mode is interlaced */
-#define IS_MODEX 4		/* ModeX style 256 colors */
-#define IS_DYNAMICMODE 8	/* Dynamic defined mode */
-#define CAPABLE_LINEAR 16	/* Can go to linear addressing mode. */
-#define IS_LINEAR 32		/* Linear addressing enabled. */
-#define EXT_INFO_AVAILABLE 64	/* Returned modeinfo contains valid extended fields */
-#define RGB_MISORDERED 128	/* Mach32 32bpp uses 0BGR instead of BGR0. */
-    /* As of this version 1.25 also used to signal if real RGB
-       (red first in memory) is used instead of BGR (Mach32 DAC 4) */
-#define HAVE_EXT_SET 256	/* vga_ext_set() available */
+#define HAVE_RWPAGE 		   1	/* vga_setreadpage() / vga_setwritepage() available */
+#define IS_INTERLACED		   2	/* mode is interlaced */
+#define IS_MODEX			   4	/* ModeX style 256 colors */
+#define IS_DYNAMICMODE		   8	/* Dynamic defined mode */
+#define CAPABLE_LINEAR		  16	/* Can go to linear addressing mode. */
+#define IS_LINEAR			  32	/* Linear addressing has been set by vga_setlinearaddressing. */
+#define EXT_INFO_AVAILABLE	  64	/* Returned modeinfo contains valid extended fields */
+#define RGB_MISORDERED		 128	/* Mach32 32bpp uses 0BGR instead of BGR0. */
+#define HAVE_EXT_SET		 256	/* vga_ext_set() available */
+#define LINEAR_CAN			 512	/* Linear mode can be used (even if not set). */
+#define IOCTL_SETDISPLAY	1024	/* The card supports ioctl method for setting display
+										start at next vertical blank time */
+#define LINEAR_USE (IS_LINEAR|LINEAR_CAN)
 
-#define IS_IN_STANDARD_VGA_DRIVER(mode) ( \
-        ((mode) < G640x480x256) || ((mode) == G720x348x2) )
+/* Don't use linear in vga modes (even 320x200x256). It does not work with some
+ * cards. */
+#define CAN_USE_LINEAR ((__svgalib_modeinfo_linearset & LINEAR_USE) && !STDVGAMODE(CM))
 
     typedef struct {
 	int width;
@@ -281,6 +294,16 @@ extern "C"
 	/* depends from actual driver/chiptype.. etc. */
     } vga_modeinfo;
 
+    typedef struct {
+        int version;
+        int size;
+        int chipset;
+        long physmem;
+		long physmemsize; /* Will there be a card with more than 4GB? */
+		void * linearmem;
+    } vga_cardinfo;
+
+    extern vga_cardinfo *vga_getcardinfo(void);
     extern vga_modeinfo *vga_getmodeinfo(int mode);
     extern int vga_getdefaultmode(void);
     extern int vga_getcurrentmode(void);
@@ -304,6 +327,7 @@ extern "C"
     extern int vga_setmodeX(void);
     extern int vga_init(void);	/* Used to return void in svgalib <= 1.12. */
     extern int vga_initf(int);
+	extern void vga_norevokeprivs(void);
     extern int vga_getmousetype(void);
     extern int vga_getmonitortype(void);
     extern void vga_setmousesupport(int s);
@@ -360,7 +384,14 @@ extern "C"
 #define G400		29
 #define R128		30
 #define SAVAGE		31
-
+#define MILLENNIUM	32
+#define I810		33
+#define TRIDENT		34
+#define RENDITION 	35
+#define G450C2		36
+#define PM2			37
+#define UNICHROME	38
+    
     /* Hor. sync: */
 #define MON640_60	0	/* 31.5 KHz (standard VGA) */
 #define MON800_56	1	/* 35.1 KHz (old SVGA) */
@@ -372,6 +403,7 @@ extern "C"
 
     extern void vga_setchipset(int c);
     extern void vga_setchipsetandfeatures(int c, int par1, int par2);
+    extern void vga_disablechipset(int c);
     extern void vga_gettextfont(void *font);
     extern void vga_puttextfont(void *font);
     extern void vga_settextmoderegs(void *regs);
@@ -389,6 +421,12 @@ extern "C"
     extern int vga_ext_set(unsigned what,...);
     extern int vga_accel(unsigned operation,...);
 
+    extern int vga_initcursor(int);
+    extern void vga_showcursor(int);
+    extern void vga_setcursorposition(int, int);
+    extern void vga_selectcursor(int);
+    extern void vga_setcursorimage(int, int, int, int, unsigned char *);
+
     extern int vga_setcrtcregs(unsigned char *);
     extern int vga_getcrtcregs(unsigned char *);
 
@@ -430,6 +468,8 @@ extern "C"
 
    extern int vga_guesstiming(int x, int y, int clue, int arg);
 
+   extern void vga_dpms(int mode);
+
 /* Valid values for what in vga_ext_set: */
 #define VGA_EXT_AVAILABLE	0	/* supported flags */
 #define VGA_EXT_SET		1	/* set flag(s) */
@@ -504,6 +544,10 @@ extern "C"
 #define BLITS_SYNC			0
 #define BLITS_IN_BACKGROUND		0x1
 
+#ifdef ROP_XOR
+#undef ROP_XOR
+#endif
+
 /* Raster ops. */
 #define ROP_COPY			0	/* Straight copy. */
 #define ROP_OR				1	/* Source OR destination. */
@@ -543,6 +587,18 @@ extern void vga_runinbackground(int stat, ...);
 #define VGA_GOTOBACK -1
 #define VGA_COMEFROMBACK -2
 extern int vga_runinbackground_version(void);
+extern void vga_waitvtactive(void);
+
+#ifdef _SVGALIB_LRMI
+typedef struct {
+    int (*rm_init)(void);
+    int (*rm_call)(struct LRMI_regs *r);
+    int (*rm_int)(int interrupt, struct LRMI_regs *r);
+    void * (*rm_alloc_real)(int size);
+    void (*rm_free_real)(void *m);
+} LRMI_callbacks;
+extern int vga_set_LRMI_callbacks(LRMI_callbacks * LRMI);
+#endif
 
 #ifdef __cplusplus
 }
